/* USER CODE BEGIN Header */
/**
 ******************************************************************************
 * @file           : main.c
 * @brief          : Main program body
 ******************************************************************************
 * @attention
 *
 * Copyright (c) 2022 STMicroelectronics.
 * All rights reserved.
 *
 * This software is licensed under terms that can be found in the LICENSE file
 * in the root directory of this software component.
 * If no LICENSE file comes with this software, it is provided AS-IS.
 *
 ******************************************************************************
 */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "i2c.h"
#include "usart.h"
#include "gpio.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "DEV_Config.h"
#include "PAJ7620U2.h"
#include "ssd1306.h"
#include "ssd1306_tests.h"
#include "pca9685.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
#define SERVO_COUNT	8
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */
unsigned char i;
unsigned short Gesture_Data;
char lcd_line[32];
uint8_t ActiveServo;
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
/* USER CODE BEGIN PFP */
uint8_t PAJ7620U2_init(void);
/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
 * @brief  The application entry point.
 * @retval int
 */
int main(void)
{
	/* USER CODE BEGIN 1 */

	/* USER CODE END 1 */

	/* MCU Configuration--------------------------------------------------------*/

	/* Reset of all peripherals, Initializes the Flash interface and the Systick. */
	HAL_Init();

	/* USER CODE BEGIN Init */

	/* USER CODE END Init */

	/* Configure the system clock */
	SystemClock_Config();

	/* USER CODE BEGIN SysInit */

	/* USER CODE END SysInit */

	/* Initialize all configured peripherals */
	MX_GPIO_Init();
	MX_USART2_UART_Init();
	MX_I2C1_Init();
	MX_I2C3_Init();
	/* USER CODE BEGIN 2 */
	ssd1306_Init();
	HAL_Delay(100);
//	ssd1306_TestAll();

	if (!PAJ7620U2_init())
	{
		ssd1306_Fill(Black);
		ssd1306_SetCursor(0, 10);
		ssd1306_WriteString("PAJ7620 error", Font_7x10, White);
		ssd1306_UpdateScreen();
		return 0;
	}
	ssd1306_Fill(Black);
	ssd1306_SetCursor(0, 10);
	ssd1306_WriteString("PAJ7620 found", Font_7x10, White);
	ssd1306_UpdateScreen();

	DEV_I2C_WriteByte(PAJ_BANK_SELECT, 0);						//Select Bank 0
	for (i = 0; i < Gesture_Array_SIZE; i++)
	{
		DEV_I2C_WriteByte(Init_Gesture_Array[i][0], Init_Gesture_Array[i][1]);//Gesture register initializes

	}
	HAL_Delay(500);

	PCA9685_Init(&hi2c1);
	PCA9685_SetServoAngle(0, 0);
	PCA9685_SetServoAngle(1, 0);
	PCA9685_SetServoAngle(2, 0);
	PCA9685_SetServoAngle(3, 0);
	PCA9685_SetServoAngle(4, 0);
	PCA9685_SetServoAngle(5, 0);
	PCA9685_SetServoAngle(6, 0);
	PCA9685_SetServoAngle(7, 0);
	HAL_Delay(500);

	/* USER CODE END 2 */

	/* Infinite loop */
	/* USER CODE BEGIN WHILE */
	while (1)
	{
		Gesture_Data = DEV_I2C_ReadWord(PAJ_INT_FLAG1);
		if (Gesture_Data)
		{
			switch (Gesture_Data)
			{
			case PAJ_UP:
				sprintf(lcd_line, "Gesture: %s", "Up");
				PCA9685_SetServoAngle(0, 90);
				PCA9685_SetServoAngle(1, 0);
				PCA9685_SetServoAngle(2, 0);
				PCA9685_SetServoAngle(3, 0);
				PCA9685_SetServoAngle(4, 0);
				PCA9685_SetServoAngle(5, 0);
				PCA9685_SetServoAngle(6, 0);
				PCA9685_SetServoAngle(7, 0);
				break;
			case PAJ_DOWN:
				sprintf(lcd_line, "Gesture: %s", "Down");
				PCA9685_SetServoAngle(0, 0);
				PCA9685_SetServoAngle(1, 90);
				PCA9685_SetServoAngle(2, 0);
				PCA9685_SetServoAngle(3, 0);
				PCA9685_SetServoAngle(4, 0);
				PCA9685_SetServoAngle(5, 0);
				PCA9685_SetServoAngle(6, 0);
				PCA9685_SetServoAngle(7, 0);
				break;
			case PAJ_LEFT:
				sprintf(lcd_line, "Gesture: %s", "Left");
				PCA9685_SetServoAngle(0, 0);
				PCA9685_SetServoAngle(1, 0);
				PCA9685_SetServoAngle(2, 90);
				PCA9685_SetServoAngle(3, 0);
				PCA9685_SetServoAngle(4, 0);
				PCA9685_SetServoAngle(5, 0);
				PCA9685_SetServoAngle(6, 0);
				PCA9685_SetServoAngle(7, 0);
				break;
			case PAJ_RIGHT:
				sprintf(lcd_line, "Gesture: %s", "Right");
				PCA9685_SetServoAngle(0, 0);
				PCA9685_SetServoAngle(1, 0);
				PCA9685_SetServoAngle(2, 0);
				PCA9685_SetServoAngle(3, 90);
				PCA9685_SetServoAngle(4, 0);
				PCA9685_SetServoAngle(5, 0);
				PCA9685_SetServoAngle(6, 0);
				PCA9685_SetServoAngle(7, 0);
				break;
			case PAJ_FORWARD:
				sprintf(lcd_line, "Gesture: %s", "Forward");
				PCA9685_SetServoAngle(0, 0);
				PCA9685_SetServoAngle(1, 0);
				PCA9685_SetServoAngle(2, 0);
				PCA9685_SetServoAngle(3, 0);
				PCA9685_SetServoAngle(4, 90);
				PCA9685_SetServoAngle(5, 0);
				PCA9685_SetServoAngle(6, 0);
				PCA9685_SetServoAngle(7, 0);
				break;
			case PAJ_BACKWARD:
				sprintf(lcd_line, "Gesture: %s", "Backward");
				PCA9685_SetServoAngle(0, 0);
				PCA9685_SetServoAngle(1, 0);
				PCA9685_SetServoAngle(2, 0);
				PCA9685_SetServoAngle(3, 0);
				PCA9685_SetServoAngle(4, 0);
				PCA9685_SetServoAngle(5, 90);
				PCA9685_SetServoAngle(6, 0);
				PCA9685_SetServoAngle(7, 0);
				break;
			case PAJ_CLOCKWISE:
				sprintf(lcd_line, "Gesture: %s", "CW");
				PCA9685_SetServoAngle(0, 0);
				PCA9685_SetServoAngle(1, 0);
				PCA9685_SetServoAngle(2, 0);
				PCA9685_SetServoAngle(3, 0);
				PCA9685_SetServoAngle(4, 0);
				PCA9685_SetServoAngle(5, 0);
				PCA9685_SetServoAngle(6, 90);
				PCA9685_SetServoAngle(7, 0);
				break;
			case PAJ_COUNT_CLOCKWISE:
				sprintf(lcd_line, "Gesture: %s", "CCW");
				PCA9685_SetServoAngle(0, 0);
				PCA9685_SetServoAngle(1, 0);
				PCA9685_SetServoAngle(2, 0);
				PCA9685_SetServoAngle(3, 0);
				PCA9685_SetServoAngle(4, 0);
				PCA9685_SetServoAngle(5, 0);
				PCA9685_SetServoAngle(6, 0);
				PCA9685_SetServoAngle(7, 90);
				break;
			case PAJ_WAVE:
				sprintf(lcd_line, "Gesture: %s", "Wave");
				break;
			default:
				break;
			}

			ssd1306_Fill(Black);
			ssd1306_SetCursor(0, 30);
			ssd1306_WriteString(lcd_line, Font_7x10, White);
			ssd1306_UpdateScreen();
			Gesture_Data = 0;

			DEV_Delay_ms(50);
		}
		/* USER CODE END WHILE */

		/* USER CODE BEGIN 3 */
	}
	/* USER CODE END 3 */
}

/**
 * @brief System Clock Configuration
 * @retval None
 */
void SystemClock_Config(void)
{
	RCC_OscInitTypeDef RCC_OscInitStruct =
	{ 0 };
	RCC_ClkInitTypeDef RCC_ClkInitStruct =
	{ 0 };

	/** Configure the main internal regulator output voltage
	 */
	__HAL_RCC_PWR_CLK_ENABLE();
	__HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

	/** Initializes the RCC Oscillators according to the specified parameters
	 * in the RCC_OscInitTypeDef structure.
	 */
	RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
	RCC_OscInitStruct.HSIState = RCC_HSI_ON;
	RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
	RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
	RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
	RCC_OscInitStruct.PLL.PLLM = 16;
	RCC_OscInitStruct.PLL.PLLN = 336;
	RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV4;
	RCC_OscInitStruct.PLL.PLLQ = 4;
	if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
	{
		Error_Handler();
	}

	/** Initializes the CPU, AHB and APB buses clocks
	 */
	RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_SYSCLK
			| RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2;
	RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
	RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
	RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
	RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

	if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
	{
		Error_Handler();
	}
}

/* USER CODE BEGIN 4 */
uint8_t PAJ7620U2_init()
{
	uint8_t i, State;
	DEV_Set_I2CAddress(PAJ7620U2_I2C_ADDRESS);
	DEV_Delay_ms(5);
	State = DEV_I2C_ReadByte(0x00);									//Read State
	if (State != 0x20)
		return 0;												//Wake up failed
	DEV_I2C_WriteByte(PAJ_BANK_SELECT, 0);						//Select Bank 0
	for (i = 0; i < Init_Array; i++)
	{
		DEV_I2C_WriteByte(Init_Register_Array[i][0], Init_Register_Array[i][1]);//Power up initialize
	}
	return 1;
}
/* USER CODE END 4 */

/**
 * @brief  This function is executed in case of error occurrence.
 * @retval None
 */
void Error_Handler(void)
{
	/* USER CODE BEGIN Error_Handler_Debug */
	/* User can add his own implementation to report the HAL error return state */
	__disable_irq();
	while (1)
	{
	}
	/* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
